<?php
/**
 * BSeller - B2W Companhia Digital
 *
 * DISCLAIMER
 *
 * Do not edit this file if you want to update this module for future new versions.
 *
 * @copyright     Copyright (c) 2017 B2W Companhia Digital. (http://www.bseller.com.br/)
 * Access https://ajuda.skyhub.com.br/hc/pt-br/requests/new for questions and other requests.
 */

namespace B2W\SkyHub\Model\Validation;

use B2W\SkyHub\Contract\Entity\Product\VariationEntityInterface;
use B2W\SkyHub\Contract\Entity\ProductEntityInterface;
use B2W\SkyHub\Exception\Integrator\Catalog\Product\Validation\AttributeRequiredException;
use B2W\SkyHub\Exception\Integrator\Catalog\Product\Validation\DownloadableProductException;
use B2W\SkyHub\Exception\Integrator\Catalog\Product\Validation\VirtualProductException;

/**
 * Class ProductEntityValidator
 * @package B2W\SkyHub\Model\Validation
 */
class ProductEntityValidator
{
    /**
     * @param ProductEntityInterface $product
     * @return bool
     * @throws AttributeRequiredException
     * @throws DownloadableProductException
     * @throws VirtualProductException
     */
    public function validate(ProductEntityInterface $product)
    {
        if (!$product->getId()) {
            $this->throwAttributeError('id', $product);
        }

        if (!$product->getSku()) {
            $this->throwAttributeError('sku', $product);
        }

        if ($product->getAdditional('_virtual') == 'yes') {
            throw new VirtualProductException();
        }

        if ($product->getAdditional('_downloadable') == 'yes') {
            throw new DownloadableProductException();
        }

        $this->_validateAttributes($product);

        $this->_validateVariations($product);

        return true;
    }

    /**
     * @param ProductEntityInterface $product
     * @return bool
     * @throws AttributeRequiredException
     */
    protected function _validateAttributes(ProductEntityInterface $product)
    {
        $attributes = \App::config('catalog/product/attribute/skyhub');

        foreach ($attributes as $attr) {
            if (!isset($attr['required']) || $attr['required'] != true) {
                continue;
            }

            $method = 'get' . ucfirst($attr['code']);

            if (!method_exists($product, $method)) {
                self::throwAttributeError($attr['code'], $product);
            }

            $value = $product->$method();

            if (!$value) {
                self::throwAttributeError($attr['code'], $product);
            }
        }

        return true;
    }

    /**
     * @param ProductEntityInterface $product
     * @return bool
     * @throws AttributeRequiredException
     */
    protected function _validateVariations(ProductEntityInterface $product)
    {
        if (!$product->getVariations()) {
            return true;
        }

        /** @var VariationEntityInterface $variation */
        foreach ($product->getVariations() as $variation) {

            $sku = $variation->getSku();

            if (empty($sku)) {
                throw new AttributeRequiredException(
                    'Variation #'.$variation->getId().' SKU can not be empty' . ' - ID ' . $product->getId()
                );
            }
        }

        return true;
    }

    /**
     * @param $attribute
     * @param ProductEntityInterface $product
     * @throws AttributeRequiredException
     */
    protected function throwAttributeError($attribute, ProductEntityInterface $product)
    {
        $message = 'Attribute ' . $attribute . ' is required for product ' . $product->getSku() . ' - ID ' . $product->getId();
        throw new AttributeRequiredException($message);
    }
}
